pub struct GraphLoader;Implementations
Sourceimpl GraphLoader
impl GraphLoader
Sourcepub fn load_from_str(yaml: &str) -> Result<StateGraph>
pub fn load_from_str(yaml: &str) -> Result<StateGraph>
Load a graph from YAML string
LLM nodes will be created as placeholders that log but don’t execute.
Use load_from_str_with_context to provide a model provider for
functional LLM nodes.
Sourcepub fn load_from_str_with_context(
yaml: &str,
ctx: &GraphLoaderContext,
) -> Result<StateGraph>
pub fn load_from_str_with_context( yaml: &str, ctx: &GraphLoaderContext, ) -> Result<StateGraph>
Load a graph from YAML string with runtime context
If the context contains a model provider, LLM nodes will be created as functional nodes that execute actual LLM calls.