GraphLoaderContext

Struct GraphLoaderContext 

pub struct GraphLoaderContext {
    pub provider: Option<Arc<dyn ModelProvider>>,
    pub default_model: Option<String>,
}
Context for loading graphs with runtime dependencies

Provides optional access to model providers for creating functional LLM nodes. When no provider is set, LLM nodes will be created as placeholders that log their invocation but don’t make actual LLM calls.

Fields

provider: Option<Arc<dyn ModelProvider>>

Model provider for LLM nodes

default_model: Option<String>

Agent-level default model pin used in precedence resolution.

Implementations

Source

impl GraphLoaderContext

pub fn new() -> Self

Create a new empty context

pub fn with_provider(provider: Arc<dyn ModelProvider>) -> Self

Create a context with a model provider

pub fn with_provider_and_model( provider: Arc<dyn ModelProvider>, default_model: impl Into<String>, ) -> Self

Create a context with provider and agent-level default model.