reduce

Function reduce 

pub fn reduce(
    execution: &mut Execution,
    action: ExecutionAction,
) -> Result<(), ReducerError>
Apply an action to an execution, producing a new state

This is a pure function - it doesn’t mutate the execution in place, but returns whether the transition was valid and applies it if so.