pub struct ExecutionSnapshot {
pub execution_id: ExecutionId,
pub tenant_id: TenantId,
pub user_id: Option<UserId>,
pub state: ExecutionState,
pub current_step_id: Option<StepId>,
pub step_outputs: HashMap<StepId, Value>,
pub variables: HashMap<String, Value>,
pub timestamp: DateTime<Utc>,
pub sequence_number: u64,
}Expand description
Execution state snapshot
This captures the current state of an execution for fast resumption.
The sequence_number should match the EventStore sequence to verify freshness.
Fields
execution_id: ExecutionIdThe execution this snapshot is for
tenant_id: TenantIdTenant for multi-tenancy isolation
user_id: Option<UserId>User who initiated this execution (for observability)
state: ExecutionStateCurrent execution state
current_step_id: Option<StepId>Currently executing step (if any)
step_outputs: HashMap<StepId, Value>Outputs from completed steps
variables: HashMap<String, Value>Working variables
timestamp: DateTime<Utc>When this snapshot was created
sequence_number: u64EventStore sequence number at snapshot time
Used to verify freshness - if EventStore has more events, the snapshot may be stale.
Implementations
Sourceimpl ExecutionSnapshot
impl ExecutionSnapshot
Sourcepub fn new(
execution_id: ExecutionId,
tenant_id: TenantId,
state: ExecutionState,
sequence_number: u64,
) -> Self
pub fn new( execution_id: ExecutionId, tenant_id: TenantId, state: ExecutionState, sequence_number: u64, ) -> Self
Create a new snapshot
Sourcepub fn with_user(
execution_id: ExecutionId,
tenant_id: TenantId,
user_id: Option<UserId>,
state: ExecutionState,
sequence_number: u64,
) -> Self
pub fn with_user( execution_id: ExecutionId, tenant_id: TenantId, user_id: Option<UserId>, state: ExecutionState, sequence_number: u64, ) -> Self
Create a new snapshot with user_id