ProtectedEventEmitter

Struct ProtectedEventEmitter 

pub struct ProtectedEventEmitter { /* private fields */ }
Protected Event Emitter

Wraps EventEmitter with output processor pipeline. All events pass through protection before being added to the stream.

Usage

use enact_core::streaming::{ProtectedEventEmitter, PiiProtectionProcessor};

let emitter = ProtectedEventEmitter::new()
    .with_processor(Arc::new(PiiProtectionProcessor::new()))
    .with_context(ProtectionContext::for_stream());

Implementations

Source

impl ProtectedEventEmitter

pub fn new() -> Self

Create a new protected event emitter

pub fn with_mode(mode: StreamMode) -> Self

Create with a specific stream mode

pub fn with_processor(self, processor: Arc<dyn OutputProcessor>) -> Self

Add a processor to the pipeline

pub fn with_context(self, context: ProtectionContext) -> Self

Set the protection context

pub fn set_context(&mut self, context: ProtectionContext)

Set the protection context (mutable)

pub fn context(&self) -> &ProtectionContext

Get the protection context

pub async fn emit(&self, event: StreamEvent) -> Result<()>

Emit an event (runs through protection pipeline)

This is async because processors may need async operations

pub fn emit_unprotected(&self, event: StreamEvent)

Emit an event synchronously (bypasses protection pipeline)

Use only for events that are guaranteed safe (control events, etc.)

pub async fn emit_force(&self, event: StreamEvent) -> Result<()>

Emit an event unconditionally (ignores mode, runs through protection)

pub fn drain(&self) -> Vec<StreamEvent>

Get all collected events

pub fn mode(&self) -> StreamMode

Get the current stream mode

pub fn inner(&self) -> &EventEmitter

Get reference to inner emitter (for compatibility)