pub trait ExecutionCallbacks:
BeforeAgentCallback
+ AfterAgentCallback
+ BeforeModelCallback
+ AfterModelCallback
+ BeforeToolCallback
+ AfterToolCallback { }Expand description
Combined callback trait for all execution events
Implement this trait to handle all callback types with a single implementation. This is useful for telemetry backends that need to track all execution events.